from kupy.config import *
from kupy.logger import logger


def test_properties_been_load():
    logger.info(type(default_setting))
    assert configs is not None
    assert configs.get("sqlalchemy_db_string").data != ""
    assert configs.get("log_level").data != ""
    assert configs.get("something_not_exists") is None



from kupy.logger import logger


def test_logger():
    assert logger is not None
    logger.info("This is info level message")
    logger.debug("This is debug level message")
    logger.warning("This is warning level message")
    logger.error("This is warning level message")



import datetime
import pytest
from kupy.dbadaptor import DBAdaptor
from kupy.logger import logger
from kupy.config import configs
import os

from tests.domain import Fund, SyncStatus

given = pytest.mark.parametrize
skipif = pytest.mark.skipif
skip = pytest.mark.skip
xfail = pytest.mark.xfail


query_sql = "select * from fund"
expect_cache_file_path = (
    configs["data_folder"].data
    + "cache/"
    + DBAdaptor.get_hash_filename(query_sql)
    + ".pkl"
)


class TestDBAdaptor:
    @pytest.fixture(scope="class")
    def db(self):
        logger.info("Setup for Class")
        db = DBAdaptor(is_use_cache=True)
        db.execute_sql_file(
            os.path.dirname(os.path.abspath(__file__)) + "/init.sql"
        )
        if os.path.exists(expect_cache_file_path):
            os.remove(expect_cache_file_path)
        return db

    def test_delete_by_id_range(self, db: DBAdaptor):
        # Insert two records to fund
        entity_list = list()
        for i in range(5):
            fund = Fund()
            fund.ticker = "40000" + str(i)
            entity_list.append(fund)
        db.save_all(entity_list)

    def test_cache_file_name(self, db):
        cache1 = db.get_hash_filename("select * from fund")
        assert 5 == len(cache1)
        cache2 = db.get_hash_filename("select * from sync_status")
        assert 5 == len(cache2)
        assert cache1 != cache2

    def test_get_sql_without_cache(self, db):
        db.set_cache_mode(False)
        df = db.get_df_by_sql(query_sql)
        assert df is not None

    def test_get_sql_with_cache(self, db):
        db.set_cache_mode(True)
        df, csv_file = db.get_df_csv_by_sql(query_sql)
        assert df is not None
        assert csv_file is not None
        assert os.path.exists(expect_cache_file_path)
        assert os.path.exists(csv_file)

    def test_update_sync_status(self, db: DBAdaptor):
        # db.set_cache_mode(True)
        assert db.update_any_by_id(
            SyncStatus,
            1,
            {
                "rc": True,
                "update_time": datetime.datetime.now(),
                "comment": "更新前:55,增量:10",
            },
        )

    def test_load_sql(self, db):
        df_equ = db.get_df_by_sql(
            "select sec_id,ticker, sec_short_name from \
            fund"
        )
        assert df_equ is not None and df_equ.shape[0] > 0
        logger.info(f"stock account:{df_equ.shape[0]}")
        ticker = "000001"
        assert (
            df_equ.loc[df_equ.ticker == ticker, "sec_short_name"].iloc[0]
            == "平安基金"
        )

    def test_update_any_by_ticker(self):
        update_dict = {
            "501216": {"list_status_cd": "L"},
            "501216": {"list_status_cd": "L"},
        }
        db = DBAdaptor()
        db.update_any_by_ticker(Fund, update_dict)
        df = db.get_df_by_sql(
            "select ticker, list_status_cd from fund where ticker in ('501216' ) "
        )
        logger.info(str(df))
        assert df.loc[df["list_status_cd"] != "L", :].shape[0] == 0

    def test_get_any_by_id(self, db: DBAdaptor):
        ss = db.get_any_by_id(SyncStatus, 1)
        assert ss is not None
        assert ss.table_name == "equity"
        assert ss.rc

    def test_execute_any_sql(self, db: DBAdaptor):
        db.execute_any_sql(
            "update sync_status set comment='update by test_execute_any_sql' where id =1 "
        )
        syncstatus = db.get_any_by_id(SyncStatus, 1)

        assert "update by test_execute_any_sql" == syncstatus.comment

    def test_get_any_by_any_column(self, db: DBAdaptor):
        ss = db.get_any_by_any_column(SyncStatus, "table_name", "equity")[0]
        # ss = db.get_any_by_id(SyncStatus, 1)
        assert ss is not None
        assert ss.table_name == "equity"
        assert ss.rc

    def test_execute_sql_file(self, db: DBAdaptor):
        test_sql_file = (
            os.path.dirname(os.path.abspath(__file__)) + "/test_sql_file.sql"
        )
        ret = db.execute_sql_file(test_sql_file)
        assert ret



from datetime import datetime

from sqlalchemy import (
    Boolean,
    Column,
    DateTime,
    Float,
    Integer,
    Sequence,
    String,
)
from sqlalchemy.ext.declarative import declarative_base

Base = declarative_base()


# sqlite don't support schema, so comment out the schema
# Use postgresql if you want to test schema and sequence
class SyncStatus(Base):
    __tablename__ = "sync_status"
    # __table_args__ = {"schema": "pyb"}

    id = Column(
        Integer,
        primary_key=True,
    )
    rc = Column(Boolean)
    table_name = Column(String(64))  #     证券交易所
    update_time = Column(DateTime)  #     日期
    comment = Column(String(255))  # 更新说明

    def __init__(self):
        pass


# 不完整字段
class Fund(Base):
    __tablename__ = "fund"
    # __table_args__ = {"schema": "pyb"}

    # sqlite don't support sequence
    id = Column(
        # Integer, Sequence("fund_id_seq", schema="pyb"), primary_key=True
        Integer,
        primary_key=True,
    )
    sec_id = Column(
        String(255)
    )  # secID; //    通联编制的证券编码，可使用DataAPI.SecIDGet获取
    ticker = Column(String(255))  # ticker; //    通用交易代码
    sec_short_name = Column(String(255))  # secShortName; //    证券简称
    category = Column(String(255))
    operation_mode = Column(String(255))
    index_fund = Column(String(255))
    etf_lof = Column(String(255))
    is_qdii = Column(String(255))
    is_fof = Column(String(255))
    exchange_cd = Column(String(255))
    list_status_cd = Column(String(255))
    list_date = Column(DateTime)
    delist_date = Column(DateTime)
    idx_id = Column(String(255))
    idx_ticker = Column(String(255))
    idx_short_name = Column(String(255))

    def __init__(self):
        pass
